@@ -345,10 +345,8 @@ static void print_nh_res_bucket(FILE *fp, const struct rtattr *res_bucket_attr)
 
 static void ipnh_destroy_entry(struct nh_entry *nhe)
 {
-	if (nhe->nh_encap)
-		free(nhe->nh_encap);
-	if (nhe->nh_groups)
-		free(nhe->nh_groups);
+	free(nhe->nh_encap);
+	free(nhe->nh_groups);
 }
 
 /* parse nhmsg into nexthop entry struct which must be destroyed by
@@ -586,8 +584,7 @@ static struct nh_entry *ipnh_cache_add(__u32 nh_id)
 	ipnh_cache_link_entry(nhe);
 
 out:
-	if (answer)
-		free(answer);
+	free(answer);
 
 	return nhe;
 
@@ -1021,6 +1018,7 @@ static int ipnh_get_id(__u32 id)
 	new_json_obj(json);
 
 	if (print_nexthop_nocache(answer, (void *)stdout) < 0) {
+		delete_json_obj();
 		free(answer);
 		return -1;
 	}
@@ -1106,6 +1104,7 @@ static int ipnh_list_flush(int argc, char **argv, int action)
 	new_json_obj(json);
 
 	if (rtnl_dump_filter(&rth, print_nexthop_nocache, stdout) < 0) {
+		delete_json_obj();
 		fprintf(stderr, "Dump terminated\n");
 		return -2;
 	}
@@ -1181,6 +1180,7 @@ static int ipnh_bucket_list(int argc, char **argv)
 	new_json_obj(json);
 
 	if (rtnl_dump_filter(&rth, print_nexthop_bucket, stdout) < 0) {
+		delete_json_obj();
 		fprintf(stderr, "Dump terminated\n");
 		return -2;
 	}
@@ -1221,6 +1221,7 @@ static int ipnh_bucket_get_id(__u32 id, __u16 bucket_index)
 	new_json_obj(json);
 
 	if (print_nexthop_bucket(answer, (void *)stdout) < 0) {
+		delete_json_obj();
 		free(answer);
 		return -1;
 	}
