@@ -108,7 +108,7 @@ static int get_hex(char c)
 	return -1;
 }
 
-int get_integer(int *val, const char *arg, int base)
+int get_long(long *val, const char *arg, int base)
 {
 	long res;
 	char *ptr;
@@ -133,6 +133,18 @@ int get_integer(int *val, const char *arg, int base)
 	if ((res == LONG_MAX || res == LONG_MIN) && errno == ERANGE)
 		return -1;
 
+	if (val)
+		*val = res;
+	return 0;
+}
+
+int get_integer(int *val, const char *arg, int base)
+{
+	long res;
+
+	if (get_long(&res, arg, base) < 0)
+		return -1;
+
 	/* Outside range of int */
 	if (res < INT_MIN || res > INT_MAX)
 		return -1;
@@ -985,9 +997,8 @@ const char *rt_addr_n2a_r(int af, int len,
 			return inet_ntop(AF_INET6, &sa->sin6.sin6_addr,
 					 buf, buflen);
 		}
-
-		/* fallthrough */
 	}
+		/* fallthrough */
 	default:
 		return "???";
 	}
@@ -1399,6 +1410,12 @@ char *int_to_str(int val, char *buf)
 	return buf;
 }
 
+char *uint_to_str(unsigned int val, char *buf)
+{
+	sprintf(buf, "%u", val);
+	return buf;
+}
+
 int get_guid(__u64 *guid, const char *arg)
 {
 	unsigned long tmp;
