@@ -1,13 +1,8 @@
+/* SPDX-License-Identifier: GPL-2.0-or-later */
 /*
  * m_estimator.c	Parse/print estimator module options.
  *
- *		This program is free software; you can u32istribute it and/or
- *		modify it under the terms of the GNU General Public License
- *		as published by the Free Software Foundation; either version
- *		2 of the License, or (at your option) any later version.
- *
  * Authors:	Alexey Kuznetsov, <kuznet@ms2.inr.ac.ru>
- *
  */
 
 #include <stdio.h>
@@ -57,7 +52,7 @@ int parse_estimator(int *p_argc, char ***p_argv, struct tc_estimator *est)
 		return -1;
 	}
 	if (show_raw)
-		fprintf(stderr, "[estimator i=%u e=%u]\n", est->interval, est->ewma_log);
+		fprintf(stderr, "[estimator i=%hhd e=%u]\n", est->interval, est->ewma_log);
 	*p_argc = argc;
 	*p_argv = argv;
 	return 0;
