@@ -1,22 +1,10 @@
+/* SPDX-License-Identifier: GPL-2.0 */
 /*
  * m_skbedit.c		SKB Editing module
  *
  * Copyright (c) 2008, Intel Corporation.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms and conditions of the GNU General Public License,
- * version 2, as published by the Free Software Foundation.
- *
- * This program is distributed in the hope it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
- * more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses>.
- *
  * Authors:	Alexander Duyck <alexander.h.duyck@intel.com>
- *
  */
 
 #include <stdio.h>
@@ -198,8 +186,9 @@ static int print_skbedit(struct action_util *au, FILE *f, struct rtattr *arg)
 	__u16 ptype;
 	struct tc_skbedit *p;
 
+	print_string(PRINT_ANY, "kind", "%s ", "skbedit");
 	if (arg == NULL)
-		return -1;
+		return 0;
 
 	parse_rtattr_nested(tb, TCA_SKBEDIT_MAX, arg);
 
@@ -209,8 +198,6 @@ static int print_skbedit(struct action_util *au, FILE *f, struct rtattr *arg)
 	}
 	p = RTA_DATA(tb[TCA_SKBEDIT_PARMS]);
 
-	print_string(PRINT_ANY, "kind", "%s ", "skbedit");
-
 	if (tb[TCA_SKBEDIT_QUEUE_MAPPING] != NULL) {
 		print_uint(PRINT_ANY, "queue_mapping", "queue_mapping %u",
 			   rta_getattr_u16(tb[TCA_SKBEDIT_QUEUE_MAPPING]));
@@ -254,7 +241,7 @@ static int print_skbedit(struct action_util *au, FILE *f, struct rtattr *arg)
 
 	print_action_control(f, " ", p->action, "");
 
-	print_string(PRINT_FP, NULL, "%s", _SL_);
+	print_nl();
 	print_uint(PRINT_ANY, "index", "\t index %u", p->index);
 	print_int(PRINT_ANY, "ref", " ref %d", p->refcnt);
 	print_int(PRINT_ANY, "bind", " bind %d", p->bindcnt);
@@ -267,7 +254,7 @@ static int print_skbedit(struct action_util *au, FILE *f, struct rtattr *arg)
 		}
 	}
 
-	print_string(PRINT_FP, NULL, "%s", _SL_);
+	print_nl();
 
 	return 0;
 }
